// STL_Examples.cpp : Defines the entry point for the console application.
//

#include <iostream>
#include <fstream>
#include <vector>
#include <algorithm>
#include <string>
#include <list>
#include <map>
#include <deque>

using namespace std;


void Example_1( void )
{
	vector< int > theIntVector;

	// Enter something to the vector
	for( int i = 0; i < 10; ++ i )
	{
		theIntVector.push_back( /*rand()*/i );
	}

	// Sort the elements
	//			iterator start		iterator end
	sort( theIntVector.begin(), theIntVector.end() );

	// Find something in the collection
	int min_value = * min_element( theIntVector.begin(), theIntVector.end() );
	cout << "The min value = " << min_value << endl;

	int max_value = * max_element( theIntVector.begin(), theIntVector.end() );
	cout << "The max value = " << max_value << endl;

	// Find specific values
	bool _2_found = find( theIntVector.begin(), theIntVector.end(), 2 ) != theIntVector.end();
	cout << ( _2_found == true ? " 2 is " : "2 is not" ) << " in the collection" << endl;
	
	bool _22_found = find( theIntVector.begin(), theIntVector.end(), 22 ) != theIntVector.end();
	cout << ( _22_found == true ? " 22 is " : "22 is not" ) << " in the collection" << endl;


	// Copy and iterate 
	// Version I
	cout << endl << "Now we stream out the whole collection (simple version I)" << endl;
	for( int j = 0; j < theIntVector.size(); ++ j )
		cout << theIntVector[ j ] << " ";

	// Version II
	cout << endl << "Now we stream out the whole collection (version II with iterators)" << endl;
	for( vector<int>::iterator theIntVector_iterator = theIntVector.begin(); theIntVector_iterator < theIntVector.end(); ++ theIntVector_iterator )
		cout << * theIntVector_iterator << " ";

	// Version III
	cout << endl << "Now we stream out the whole collection (version III with copy)" << endl;
	copy( theIntVector.begin(), theIntVector.end(), ostream_iterator< int >( cout, ", " ) );
	// ... do the same but omit duplicates
	cout << endl << "Now we stream out the whole collection without duplicates (version III with copy)" << endl;
	unique_copy( theIntVector.begin(), theIntVector.end(), ostream_iterator< int >( cout, "; " ) );

	// The last technique (i.e. Version III) is really versatile
	// ... copy in a reversed order
	cout << endl << "Now we stream out the whole collection in REVERSE without duplicates (version III with copy)" << endl;
	copy( theIntVector.rbegin(), theIntVector.rend(), ostream_iterator< int >( cout, ", " ) );
	cout << endl << "Now we stream out the whole collection in REVERSE without duplicates (version III with copy)" << endl;
	unique_copy( theIntVector.rbegin(), theIntVector.rend(), ostream_iterator< int >( cout, "; " ) );

	// By the same token we can easily stream out to a file, look...
	ofstream myOutFile( "StudentFileName.txt" );
	if( myOutFile.is_open() != false )
	{
		myOutFile << "These are vector element in normal order" << endl;
		copy( theIntVector.begin(), theIntVector.end(), ostream_iterator< int >( myOutFile, ", " ) );

		myOutFile << "These are vector element in reversed order" << endl;
		copy( theIntVector.rbegin(), theIntVector.rend(), ostream_iterator< int >( myOutFile, "=> " ) );
	}


}


void Example_2( void )
{
	///////////////////////////////////////////////////////////
	// Let's create the list of int's
	list< int > theIntList;

	// Insert something to the list
	for( int i = 0; i < 10; ++ i )
	{
		theIntList.push_front( - i );
		theIntList.push_back( i );
	}

	// Now print data stored in the list
	cout << endl << "Now we stream out the whole collection " << endl;
	copy( theIntList.begin(), theIntList.end(), ostream_iterator< int >( cout, ", " ) );

	///////////////////////////////////////////////////////////
	// Let's create the other container and copy there the list
	vector< int > theIntVector;

	// ... but before we copy we have to be sure there is enough room 
	cout << endl << "Vector size before resize: " << theIntVector.size() << endl;
	theIntVector.resize( theIntList.size() );
	cout << endl << "Vector size after resize: " << theIntVector.size() << endl;

	copy( theIntList.begin(), theIntList.end(), theIntVector.begin() );	

	cout << endl << "Now we stream out the whole collection VECTOR " << endl;
	copy( theIntVector.begin(), theIntVector.end(), ostream_iterator< int >( cout, ", " ) );

	
	///////////////////////////////////////////////////////////
	// Let's create yet another container and copy there the list
	deque< int > theIntDeque( theIntVector.size() );

	copy( theIntVector.begin(), theIntVector.end(), theIntDeque.begin() );	

	theIntDeque.pop_front();	// remove one element from the head
	theIntDeque.pop_back();		// and one from the tail

	cout << endl << "Now we stream out the whole collection DEQUE: " << endl;
	copy( theIntDeque.begin(), theIntDeque.end(), ostream_iterator< int >( cout, ", " ) );



}

// Map usage
void Example_3( void )
{
	typedef map< string, int > NameAge_Map;

	NameAge_Map theDataBase;

	// Now we can insert some elements into the map
	// Does it behave like an ordianry array?
	// A new index is taken as a reason to create and to insert a new element
	// of the map.
	theDataBase[ "Kowalski" ] = 24;		// i.e. Kowalski is 24 years old...
	theDataBase[ "Zamarski" ] = 5;	
	theDataBase[ "Cyganek" ] = 35;	

	theDataBase[ "Cyganek" ] = 45;		// ???


	NameAge_Map::iterator theMapIter;

	cout << endl << endl << "====================== THE MAP ==========================" << endl;
	for( theMapIter = theDataBase.begin(); theMapIter != theDataBase.end(); ++ theMapIter )
		cout << "Key: " << theMapIter->first << ", value: " << theMapIter->second << endl;


}



int main(int argc, char* argv[])
{
	Example_1();

	Example_2();

	Example_3();

	cout << endl << "Is OK :)" << endl;


	int dummy;
	cin >> dummy;		// wait for a user

	return 0;
}

